#! /bin/sh 

# THE PATH FOR THE NJML-DIRECTORY (RESTORED FROM THE ARGUMENT)
NJMLDIR=$1

TCL_H_PATH=$2
TCL_LIB_PATH=$3
TCL_LIB=$4
TK_H_PATH=$5
TK_LIB_PATH=$6
TK_LIB=$7

RUNTIMEDIR=$NJMLDIR/src/runtime
CLIBDIR=$RUNTIMEDIR/c-libs
OBJDIR=$RUNTIMEDIR/objs
BINDIR=$NJMLDIR/bin
RUNDIR=$NJMLDIR/bin/.run

CONFIGDIR=$PWD


#
# Figure out system hardware (ARCH), operating system (OPSYS) 
# and heap suffix (HEAP_SUFFIX)
# 
echo "sorting out configuration ..."
ARCH_N_OPSYS=`$BINDIR/.arch-n-opsys`
if [ "$?" != "0" ]; then
  echo "$CMD: unable to determine architecture/operating system"
  exit 1
fi
eval $ARCH_N_OPSYS


#
# ADDING ADDITIONAL SOURCES TO NJML
# =================================
# move new library to c-libs directory
echo "adding additional sources to njml ..."
cp -r tksml/ $CLIBDIR/
#assuring links to tcl/tk sources
if [ ! -f $CLIBDIR/tksml/tcl.h ]; then
   ln -fs $TCL_H_PATH $CLIBDIR/tksml/tcl.h
   ln -fs $TK_H_PATH  $CLIBDIR/tksml/tk.h
fi


#
# PATCHING MAKEFILES
# =================================
# the ml-runtime sytem has one global and one library makefile
# save the old ones 
echo "patching makefiles in njml ..."
if [ ! -f $OBJDIR/makefile.orig ]; then
   cp $OBJDIR/makefile $OBJDIR/makefile.orig
   cp $CLIBDIR/makefile $CLIBDIR/makefile.orig
fi
# the saving of the original runtime-system 
#   mv $RUNDIR/run.<system> $RUNDIR/run.<system>
# is done below.
# move new makefiles to target directories
chmod 644 $CLIBDIR/mak*
chmod 644 $OBJDIR/mak*
chmod 644 $CLIBDIR/*.c
chmod 644 $CLIBDIR/*.h
cp configure/makefile.objs $OBJDIR/makefile
cp configure/makefile.clibs $CLIBDIR/makefile

# ANNOUNCE THE NEW LIBRARY TKSML IN CLIB-LIST.H
# ================================================
echo "announce the new library tksml in clib-list.h"
echo "C_LIBRARY(TKSML_Library)" >> $CLIBDIR/clib-list.h                      


#
# PATCHING LINKING-CONFIGURATION "mk.$HEAP_SUFFIX"
# ================================================
echo "patching link-configuration mk.$HEAP_SUFFIX"
cd $OBJDIR  
if [ ! -f mk.$HEAP_SUFFIX.orig ]; then
   cp mk.$HEAP_SUFFIX mk.$HEAP_SUFFIX.orig
fi
sed -e "s|^LD_LIBS *=|LD_LIBS = -lX11 -L$TCL_LIB_PATH -L$TK_LIB_PATH -l$TCL_LIB -l$TK_LIB |" mk.$HEAP_SUFFIX > mk.$HEAP_SUFFIX.new
mv -f mk.$HEAP_SUFFIX.new mk.$HEAP_SUFFIX
# ... AND MAKE THE SUCKER !!!
make MAKE=make -f mk.$HEAP_SUFFIX
if [ ! -f $RUNDIR/run.$HEAP_SUFFIX.orig ]; then 
   mv $RUNDIR/run.$HEAP_SUFFIX $RUNDIR/run.$HEAP_SUFFIX.orig
fi
cp run.$HEAP_SUFFIX $RUNDIR/
## make MAKE=make clean


