/* **************************************************************************

   $Source: /repository/sml/njml_patch/tksml/tksml.c,v $

   The Core of TK_INSIDE - the configuration of sml_tk 
   witk linked-in Tk-interpreter for faster communinication.
     

   $Date: 2000/04/19 10:51:38 $
   $Revision: 1.4 $

   Author: bu&behrends (Last modification $Author: bu $)

   (C) 1998, Albert-Ludwigs-Universitaet Freiburg
 
   ************************************************************************* */

#include "ml-base.h"
#include "ml-values.h"
#include "ml-objects.h"
#include "cfun-proto-list.h"
#include "tksml.h"
#include <stdio.h>



/***************************************************
 * add eventcommand p to eventcommand-queue 
 **************************************************/

int addEventCmd(char *p) {
 int len; 
 int freeSpace;
 
 len=strlen(p);

 if (to_sml_tail>=to_sml_head) 
      freeSpace = BUFFER_SIZE-to_sml_tail+to_sml_head - 1;
 else freeSpace = to_sml_head-to_sml_tail - 1;

 if (freeSpace>len) {   /* enough bytes free */
  while (*p) {
   to_sml[to_sml_tail++] = *p;
   to_sml_tail %= BUFFER_SIZE;
   p++;
  }; 
  to_sml[to_sml_tail++] = '\n';
  to_sml_tail %= BUFFER_SIZE;
  eventWasGenerated=1; /* NOTE that events generated by Tk may 
                          be empty, i.e. to_sml_tail=to_sml_head */
  /* DEBUG fprintf(stderr,"addEventCmd:%d:%d\n",to_sml_head,to_sml_tail); */
  return 1;
 }
 else {   /* queue is full */
  fprintf(stderr,"TKSML:EventCmd-Buffer Overflow !!!\n");
  return 0;
 }
 
}

extern int EventCmd _ANSI_ARGS_((ClientData clientData,
	                         Tcl_Interp *interp, 
                                 int argc, 
                                 char *argv[]));


/***************************************************
 *  EventCmd: 
 *    puts the argument string into the to_sml buffer,
 *    resets the to_sml_head and to_sml_tail (see get_tcl),
 *    in addEventCmd the MainLoop is terminated by setting
 *    can be restarted using run_tcl
 **************************************************/

int EventCmd(dummy, interp, argc, argv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
     int argc;                           /* Number of arguments. */
     char **argv;                        /* Argument strings. */
{
  if (argc!=2){
    Tcl_AppendResult(interp, "wrong # args: should be 1", (char *) NULL);
    return TCL_ERROR;
  }
  addEventCmd(argv[1]); /* Shouldnt the error be reported ? bu */
  return TCL_OK;
}




/***************************************************
 *  init_tcl: 
 *    initialize tcl/tk,
 *    create the new command `toSML' 
 *    on the tcl-side
 *    if succesful return 1
 **************************************************/

/* _ml_TKSML_init_tcl : unit -> int
 */

ml_val_t _ml_TKSML_init_tcl (ml_state_t *msp, ml_val_t arg) {

  
  /* Reset communication buffers */
  to_sml_head = 0;
  to_sml_tail = 0;
  eventWasGenerated=0;  
 
  interp     = Tcl_CreateInterp();    /* create interpreter  */
  tclRunning = 1;                     /* interpreter created */
  
  Tcl_SetVar(interp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);
  
  if (Tk_Init(interp) == TCL_ERROR) {  /* Tk Initialization Failed? */
    fprintf(stderr, "%s\n", interp->result);
    return INT_CtoML(0);
  }
  
  
  /* New command */
  
  Tcl_CreateCommand(interp, "toSML", EventCmd, (ClientData) NULL,
		    (void (*)()) NULL);
  
  return INT_CtoML(1);
}

/***************************************************
 *  reset_tcl:
 *    destroy interpreter
***************************************************/ 
  
/* _ml_TKSML_reset_tcl : unit -> unit
*/

ml_val_t _ml_TKSML_reset_tcl (ml_state_t *msp, ml_val_t arg) 
{
  if (tclRunning == 1) Tcl_DeleteInterp(interp);
  else tclRunning = 0;
}



/***************************************************
 *  eval_tcl:
 *    evaluate the interpreter to perform action
 **************************************************/

/* _ml_TKSML_eval_tcl : string -> unit
 */

ml_val_t _ml_TKSML_eval_tcl (ml_state_t *msp, ml_val_t arg) 
{
  char *cmd = PTR_MLtoC(char,arg);
  if (Tcl_Eval(interp,cmd) != TCL_OK) {
    fprintf(stderr, "TKSML:%s\n", interp->result);
  }
} 




/***************************************************
 *  run_tcl:
 *    starts tk to handle events
 *    terminates if user interface is destroyed
 **************************************************/

/* _ml_TKSML_run_tcl : unit -> int
 */



ml_val_t  _ml_TKSML_run_tcl_nowait (ml_state_t *msp, ml_val_t arg) {
  if (to_sml_tail==to_sml_head) eventWasGenerated=0;  
  Tcl_DoOneEvent(TCL_DONT_WAIT);
  return INT_CtoML(eventWasGenerated);
}


ml_val_t  _ml_TKSML_run_tcl (ml_state_t *msp, ml_val_t arg) {
  Tcl_DoOneEvent(0);
  while (to_sml_tail==to_sml_head) Tcl_DoOneEvent(0);
  return INT_CtoML(1);
}




/*************************************************** 
 *  get_tcl:
 *    return the next character of the event buffer to sml
 *    interrupts possible !
 **************************************************/

/* _ml_TKSML_get_event : unit -> string
 */

ml_val_t  _ml_TKSML_get_event (ml_state_t *msp, ml_val_t arg) {
  char c[BUFFER_SIZE+1];
  int cnt;

  cnt= 0;
  while ((cnt < BUFFER_SIZE) /* no buffer overflow */
	 && to_sml_tail!= to_sml_head /* still sth. to read */
	 && to_sml[to_sml_head] != '\n'
	 && to_sml[to_sml_head] != 0 /* string not ended */ ) {
    c[cnt++] = to_sml[to_sml_head++];
    to_sml_head %= BUFFER_SIZE;
  }  
  if (to_sml_tail != to_sml_head) 
     to_sml_head = (to_sml_head+1)%BUFFER_SIZE; /* skip "\n" */
  c[cnt++]= '\n'; c[cnt]= 0; /* terminate string 0 */
  /* DEBUG fprintf(stderr,"get_event:%d:%d\n",to_sml_head,to_sml_tail); */
  return ML_CString(msp,c);
}

